# bsd.linux_kmod.mk to build a driver with linux emulation.
# $Id: bsd.linux_kmod.mk,v 1.5 2007/02/02 15:14:17 luigi Exp $
#
# .include file for compiling linux drivers on FreeBSD. The caller should
# set the following
#
#   .PATH:	driver-specific paths with extra sources
#   SRCS=	list of (driver) source files that we want to compile.
#   KMOD=	driver name (one word, will be used also elsewhere).
#   CFLAGS+=	any driver-specific flags you might need. Often you will need
#		-I... for subdirectories with extra headers.
#	Any other driver-specific variables.
#
#    DEV_NAME=	device name for the driver (defaults to \"video\" )
#    KMODDIR=	install location (defaults to /dev/modules)
#
# and then .include <bsd.linux_kmod.mk>

# Here we add the necessary hooks to the linux emulation layer. You
# should not change anything below this line.

# Path to the linux compat library. Should not be changed.
KLINPATH?= /usr/local/share/linux-kmod-compat/linux_compat
.PATH: ${KLINPATH}		# location of extra sources

# The extra sources we link with the linux driver
SRCS+=bus_if.h device_if.h opt_usb.h
SRCS+= ldev_stub.c linux_compat.c

DEV_NAME?=\"video\"		# default name for /dev entry
KMODDIR?=/boot/modules		# default install location

CFLAGS+= -D__KERNEL__		# compile kernel part of the linux headers
CFLAGS+= -I${KLINPATH}		# where to look for headers

# pass the driver name and device name to the driver
CFLAGS+= -DDRIVER_NAME=${KMOD} -DDEV_NAME=${DEV_NAME}

.include <bsd.kmod.mk>
